@@ -3,7 +3,7 @@
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
   This program can be distributed under the terms of the GNU GPLv2.
-  See the file GPL2.txt.
+  See the file COPYING.
 */
 
 /** @file
@@ -14,15 +14,11 @@
  *
  *     gcc -Wall hello_ll.c `pkg-config fuse3 --cflags --libs` -o hello_ll
  *
- * Note: If the pkg-config command fails due to the absence of the fuse3.pc
- *     file, you should configure the path to the fuse3.pc file in the
- *     PKG_CONFIG_PATH variable.
- *
  * ## Source code ##
  * \include hello_ll.c
  */
 
-#define FUSE_USE_VERSION FUSE_MAKE_VERSION(3, 12)
+#define FUSE_USE_VERSION 34
 
 #include <fuse_lowlevel.h>
 #include <stdio.h>
@@ -57,18 +53,6 @@ static int hello_stat(fuse_ino_t ino, struct stat *stbuf)
 	return 0;
 }
 
-static void hello_ll_init(void *userdata, struct fuse_conn_info *conn)
-{
-	(void)userdata;
-
-	/* Disable the receiving and processing of FUSE_INTERRUPT requests */
-	conn->no_interrupt = 1;
-
-	/* Test setting flags the old way */
-	conn->want = FUSE_CAP_ASYNC_READ;
-	conn->want &= ~FUSE_CAP_ASYNC_READ;
-}
-
 static void hello_ll_getattr(fuse_req_t req, fuse_ino_t ino,
 			     struct fuse_file_info *fi)
 {
@@ -173,7 +157,7 @@ static void hello_ll_getxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
 							  size_t size)
 {
 	(void)size;
-	assert(ino == 1 || ino == 2);
+	assert(ino == 2);
 	if (strcmp(name, "hello_ll_getxattr_name") == 0)
 	{
 		const char *buf = "hello_ll_getxattr_value";
@@ -190,7 +174,7 @@ static void hello_ll_setxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
 {
 	(void)flags;
 	(void)size;
-	assert(ino == 1 || ino == 2);
+	assert(ino == 2);
 	const char* exp_val = "hello_ll_setxattr_value";
 	if (strcmp(name, "hello_ll_setxattr_name") == 0 &&
 	    strlen(exp_val) == size &&
@@ -206,7 +190,7 @@ static void hello_ll_setxattr(fuse_req_t req, fuse_ino_t ino, const char *name,
 
 static void hello_ll_removexattr(fuse_req_t req, fuse_ino_t ino, const char *name)
 {
-	assert(ino == 1 || ino == 2);
+	assert(ino == 2);
 	if (strcmp(name, "hello_ll_removexattr_name") == 0)
 	{
 		fuse_reply_err(req, 0);
@@ -218,7 +202,6 @@ static void hello_ll_removexattr(fuse_req_t req, fuse_ino_t ino, const char *nam
 }
 
 static const struct fuse_lowlevel_ops hello_ll_oper = {
-	.init = hello_ll_init,
 	.lookup = hello_ll_lookup,
 	.getattr = hello_ll_getattr,
 	.readdir = hello_ll_readdir,
@@ -234,7 +217,7 @@ int main(int argc, char *argv[])
 	struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
 	struct fuse_session *se;
 	struct fuse_cmdline_opts opts;
-	struct fuse_loop_config *config;
+	struct fuse_loop_config config;
 	int ret = -1;
 
 	if (fuse_parse_cmdline(&args, &opts) != 0)
@@ -276,12 +259,9 @@ int main(int argc, char *argv[])
 	if (opts.singlethread)
 		ret = fuse_session_loop(se);
 	else {
-		config = fuse_loop_cfg_create();
-		fuse_loop_cfg_set_clone_fd(config, opts.clone_fd);
-		fuse_loop_cfg_set_max_threads(config, opts.max_threads);
-		ret = fuse_session_loop_mt(se, config);
-		fuse_loop_cfg_destroy(config);
-		config = NULL;
+		config.clone_fd = opts.clone_fd;
+		config.max_idle_threads = opts.max_idle_threads;
+		ret = fuse_session_loop_mt(se, &config);
 	}
 
 	fuse_session_unmount(se);
