@@ -5,7 +5,7 @@
   Architecture-independent mounting code.
 
   This program can be distributed under the terms of the GNU LGPLv2.
-  See the file LGPL2.txt.
+  See the file COPYING.LIB.
 */
 
 #include "fuse_config.h"
@@ -21,7 +21,7 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <paths.h>
-#if !defined( __NetBSD__) && !defined(__FreeBSD__) && !defined(__DragonFly__) && !defined(__ANDROID__)
+#if !defined( __NetBSD__) && !defined(__FreeBSD__) && !defined(__DragonFly__)
 #include <mntent.h>
 #else
 #define IGNORE_MTAB
@@ -54,6 +54,7 @@ static int mtab_needs_update(const char *mnt)
 	 * Skip mtab update if /etc/mtab:
 	 *
 	 *  - doesn't exist,
+	 *  - is a symlink,
 	 *  - is on a read-only filesystem.
 	 */
 	res = lstat(_PATH_MOUNTED, &stbuf);
@@ -64,6 +65,9 @@ static int mtab_needs_update(const char *mnt)
 		uid_t ruid;
 		int err;
 
+		if (S_ISLNK(stbuf.st_mode))
+			return 0;
+
 		ruid = getuid();
 		if (ruid != 0)
 			setreuid(0, -1);
@@ -75,10 +79,6 @@ static int mtab_needs_update(const char *mnt)
 
 		if (err == EROFS)
 			return 0;
-
-		res = access("/run/mount/utab", F_OK);
-		if (res == -1)
-			return 0;
 	}
 
 	return 1;
@@ -363,11 +363,6 @@ int fuse_mnt_parse_fuse_fd(const char *mountpoint)
 	int fd = -1;
 	int len = 0;
 
-	if (mountpoint == NULL) {
-		fprintf(stderr, "Invalid null-ptr mount-point!\n");
-		return -1;
-	}
-
 	if (sscanf(mountpoint, "/dev/fd/%u%n", &fd, &len) == 1 &&
 	    len == strlen(mountpoint)) {
 		return fd;
