@@ -3,14 +3,13 @@
   Copyright (C) 2001-2007  Miklos Szeredi <miklos@szeredi.hu>
 
   This program can be distributed under the terms of the GNU GPLv2.
-  See the file GPL2.txt.
+  See the file COPYING.
 */
 /* This program does the mounting and unmounting of FUSE filesystems */
 
-#define _GNU_SOURCE /* for clone,strchrnul and close_range */
+#define _GNU_SOURCE /* for clone and strchrnul */
 #include "fuse_config.h"
 #include "mount_util.h"
-#include "util.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,7 +24,6 @@
 #include <mntent.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <sys/param.h>
 
 #include "fuse_mount_compat.h"
 
@@ -36,18 +34,7 @@
 #include <stdbool.h>
 #include <sys/vfs.h>
 
-#if defined HAVE_CLOSE_RANGE && defined linux
-#include <linux/close_range.h>
-#endif
-
-#if defined HAVE_LISTMOUNT
-#include <linux/mount.h>
-#include <syscall.h>
-#include <stdint.h>
-#endif
-
 #define FUSE_COMMFD_ENV		"_FUSE_COMMFD"
-#define FUSE_KERN_DEVICE_ENV	"FUSE_KERN_DEVICE"
 
 #define FUSE_DEV "/dev/fuse"
 
@@ -110,75 +97,6 @@ static struct mntent *GETMNTENT(FILE *stream)
 #define GETMNTENT getmntent
 #endif // GETMNTENT_NEEDS_UNESCAPING
 
-/*
- * Take a ',' separated option string and extract "x-" options
- */
-static int extract_x_options(const char *original, char **non_x_opts,
-			     char **x_opts)
-{
-	size_t orig_len;
-	const char *opt, *opt_end;
-
-	orig_len = strlen(original) + 1;
-
-	*non_x_opts = calloc(1, orig_len);
-	*x_opts    = calloc(1, orig_len);
-
-	size_t non_x_opts_len = orig_len;
-	size_t x_opts_len = orig_len;
-
-	if (*non_x_opts == NULL || *x_opts == NULL) {
-		fprintf(stderr, "%s: Failed to allocate %zuB.\n",
-			__func__, orig_len);
-		return -ENOMEM;
-	}
-
-	for (opt = original; opt < original + orig_len; opt = opt_end + 1) {
-		char *opt_buf;
-
-		opt_end = strchr(opt, ',');
-		if (opt_end == NULL)
-			opt_end = original + orig_len;
-
-		size_t opt_len = opt_end - opt;
-		size_t opt_len_left = orig_len - (opt - original);
-		size_t buf_len;
-		bool is_x_opts;
-
-		if (strncmp(opt, "x-", MIN(2, opt_len_left)) == 0) {
-			buf_len = x_opts_len;
-			is_x_opts = true;
-			opt_buf = *x_opts;
-		} else {
-			buf_len = non_x_opts_len;
-			is_x_opts = false;
-			opt_buf = *non_x_opts;
-		}
-
-		if (buf_len < orig_len) {
-			strncat(opt_buf, ",", 2);
-			buf_len -= 1;
-		}
-
-		/* omits ',' */
-		if ((ssize_t)(buf_len - opt_len) < 0) {
-			/* This would be a bug */
-			fprintf(stderr, "%s: no buf space left in copy, orig='%s'\n",
-				__func__, original);
-			return -EIO;
-		}
-
-		strncat(opt_buf, opt, opt_end - opt);
-		buf_len -= opt_len;
-
-		if (is_x_opts)
-			x_opts_len = buf_len;
-		else
-			non_x_opts_len = buf_len;
-	}
-
-	return 0;
-}
 
 static const char *get_user_name(void)
 {
@@ -535,13 +453,11 @@ static int unmount_fuse_locked(const char *mnt, int quiet, int lazy)
 
 	drop_privs();
 	res = chdir_to_parent(copy, &last);
-	if (res == -1) {
-		restore_privs();
+	restore_privs();
+	if (res == -1)
 		goto out;
-	}
 
 	res = umount2(last, umount_flags);
-	restore_privs();
 	if (res == -1 && !quiet) {
 		fprintf(stderr, "%s: failed to unmount %s: %s\n",
 			progname, mnt, strerror(errno));
@@ -572,7 +488,7 @@ static int unmount_fuse(const char *mnt, int quiet, int lazy)
 	return res;
 }
 
-static int count_fuse_fs_mtab(void)
+static int count_fuse_fs(void)
 {
 	struct mntent *entp;
 	int count = 0;
@@ -592,72 +508,6 @@ static int count_fuse_fs_mtab(void)
 	return count;
 }
 
-#ifdef HAVE_LISTMOUNT
-static int count_fuse_fs_ls_mnt(void)
-{
-	#define SMBUF_SIZE 1024
-	#define MNT_ID_LEN 128
-
-	int fuse_count = 0;
-	int n_mounts = 0;
-	int ret = 0;
-	uint64_t mnt_ids[MNT_ID_LEN];
-	unsigned char smbuf[SMBUF_SIZE];
-	struct mnt_id_req req = {
-		.size = sizeof(struct mnt_id_req),
-	};
-	struct statmount *sm;
-
-	for (;;) {
-		req.mnt_id = LSMT_ROOT;
-
-		n_mounts = syscall(SYS_listmount, &req, &mnt_ids, MNT_ID_LEN, 0);
-		if (n_mounts == -1) {
-			if (errno != ENOSYS) {
-				fprintf(stderr, "%s: failed to list mounts: %s\n", progname,
-					strerror(errno));
-			}
-			return -1;
-		}
-
-		for (int i = 0; i < n_mounts; i++) {
-			req.mnt_id = mnt_ids[i];
-			req.param = STATMOUNT_FS_TYPE;
-			ret = syscall(SYS_statmount, &req, &smbuf, SMBUF_SIZE, 0);
-			if (ret) {
-				if (errno == ENOENT)
-					continue;
-
-				fprintf(stderr, "%s: failed to stat mount %lld: %s\n", progname,
-					req.mnt_id, strerror(errno));
-				return -1;
-			}
-
-			sm = (struct statmount *)smbuf;
-			if (sm->mask & STATMOUNT_FS_TYPE &&
-			    strcmp(&sm->str[sm->fs_type], "fuse") == 0)
-				fuse_count++;
-		}
-
-		if (n_mounts < MNT_ID_LEN)
-			break;
-		req.param = mnt_ids[MNT_ID_LEN - 1];
-	}
-	return fuse_count;
-}
-
-static int count_fuse_fs(void)
-{
-	int count = count_fuse_fs_ls_mnt();
-
-	return count >= 0 ? count : count_fuse_fs_mtab();
-}
-#else
-static int count_fuse_fs(void)
-{
-	return count_fuse_fs_mtab();
-}
-#endif
 
 #else /* IGNORE_MTAB */
 static int count_fuse_fs(void)
@@ -788,8 +638,6 @@ static struct mount_flags mount_flags[] = {
 	{"strictatime",     MS_STRICTATIME, 1, 1},
 	{"nostrictatime",   MS_STRICTATIME, 0, 1},
 	{"dirsync", MS_DIRSYNC,	    1, 1},
-	{"symfollow",       MS_NOSYMFOLLOW, 0, 1},
-	{"nosymfollow",     MS_NOSYMFOLLOW, 1, 1},
 	{NULL,	    0,		    0, 0}
 };
 
@@ -1213,19 +1061,14 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 		0x7366746E /* NTFS3_SUPER_MAGIC */,
 		0x5346414f /* OPENAFS_SUPER_MAGIC */,
 		0x794C7630 /* OVERLAYFS_SUPER_MAGIC */,
-		0xAAD7AAEA /* PANFS_SUPER_MAGIC */,
 		0x52654973 /* REISERFS_SUPER_MAGIC */,
 		0xFE534D42 /* SMB2_SUPER_MAGIC */,
 		0x73717368 /* SQUASHFS_MAGIC */,
 		0x01021994 /* TMPFS_MAGIC */,
 		0x24051905 /* UBIFS_SUPER_MAGIC */,
-		0x18031977 /* WEKAFS_SUPER_MAGIC */,
-#if __SIZEOF_LONG__ > 4
 		0x736675005346544e /* UFSD */,
-#endif
 		0x58465342 /* XFS_SB_MAGIC */,
 		0x2FC12FC1 /* ZFS_SUPER_MAGIC */,
-		0x858458f6 /* RAMFS_MAGIC */,
 	};
 	for (i = 0; i < sizeof(f_type_whitelist)/sizeof(f_type_whitelist[0]); i++) {
 		if (f_type_whitelist[i] == fs_buf.f_type)
@@ -1237,39 +1080,63 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 	return -1;
 }
 
-static int open_fuse_device(const char *dev)
+static int try_open(const char *dev, char **devp, int silent)
+{
+	int fd = open(dev, O_RDWR);
+	if (fd != -1) {
+		*devp = strdup(dev);
+		if (*devp == NULL) {
+			fprintf(stderr, "%s: failed to allocate memory\n",
+				progname);
+			close(fd);
+			fd = -1;
+		}
+	} else if (errno == ENODEV ||
+		   errno == ENOENT)/* check for ENOENT too, for the udev case */
+		return -2;
+	else if (!silent) {
+		fprintf(stderr, "%s: failed to open %s: %s\n", progname, dev,
+			strerror(errno));
+	}
+	return fd;
+}
+
+static int try_open_fuse_device(char **devp)
 {
 	int fd;
 
 	drop_privs();
-	fd = open(dev, O_RDWR);
-	if (fd == -1) {
-		if (errno == ENODEV || errno == ENOENT)/* check for ENOENT too, for the udev case */
-			fprintf(stderr,
-				"%s: fuse device %s not found. Kernel module not loaded?\n",
-				progname, dev);
-		else
-			fprintf(stderr,
-				"%s: failed to open %s: %s\n", progname, dev, strerror(errno));
-	}
+	fd = try_open(FUSE_DEV, devp, 0);
 	restore_privs();
 	return fd;
 }
 
+static int open_fuse_device(char **devp)
+{
+	int fd = try_open_fuse_device(devp);
+	if (fd >= -1)
+		return fd;
+
+	fprintf(stderr,
+		"%s: fuse device not found, try 'modprobe fuse' first\n",
+		progname);
+
+	return -1;
+}
+
+
 static int mount_fuse(const char *mnt, const char *opts, const char **type)
 {
 	int res;
 	int fd;
-	const char *dev = getenv(FUSE_KERN_DEVICE_ENV) ?: FUSE_DEV;
+	char *dev;
 	struct stat stbuf;
 	char *source = NULL;
 	char *mnt_opts = NULL;
 	const char *real_mnt = mnt;
 	int mountpoint_fd = -1;
-	char *do_mount_opts = NULL;
-	char *x_opts = NULL;
 
-	fd = open_fuse_device(dev);
+	fd = open_fuse_device(&dev);
 	if (fd == -1)
 		return -1;
 
@@ -1284,16 +1151,11 @@ static int mount_fuse(const char *mnt, const char *opts, const char **type)
 		}
 	}
 
-	// Extract any options starting with "x-"
-	res= extract_x_options(opts, &do_mount_opts, &x_opts);
-	if (res)
-		goto fail_close_fd;
-
 	res = check_perm(&real_mnt, &stbuf, &mountpoint_fd);
 	restore_privs();
 	if (res != -1)
 		res = do_mount(real_mnt, type, stbuf.st_mode & S_IFMT,
-			       fd, do_mount_opts, dev, &source, &mnt_opts);
+			       fd, opts, dev, &source, &mnt_opts);
 
 	if (mountpoint_fd != -1)
 		close(mountpoint_fd);
@@ -1308,28 +1170,6 @@ static int mount_fuse(const char *mnt, const char *opts, const char **type)
 	}
 
 	if (geteuid() == 0) {
-		if (x_opts && strlen(x_opts) > 0) {
-			/*
-			 * Add back the options starting with "x-" to opts from
-			 * do_mount. +2 for ',' and '\0'
-			 */
-			size_t mnt_opts_len = strlen(mnt_opts);
-			size_t x_mnt_opts_len =  mnt_opts_len+
-						 strlen(x_opts) + 2;
-			char *x_mnt_opts = calloc(1, x_mnt_opts_len);
-
-			if (mnt_opts_len) {
-				strcpy(x_mnt_opts, mnt_opts);
-				strncat(x_mnt_opts, ",", 2);
-			}
-
-			strncat(x_mnt_opts, x_opts,
-				x_mnt_opts_len - mnt_opts_len - 2);
-
-			free(mnt_opts);
-			mnt_opts = x_mnt_opts;
-		}
-
 		res = add_mount(source, mnt, *type, mnt_opts);
 		if (res == -1) {
 			/* Can't clean up mount in a non-racy way */
@@ -1340,8 +1180,7 @@ static int mount_fuse(const char *mnt, const char *opts, const char **type)
 out_free:
 	free(source);
 	free(mnt_opts);
-	free(x_opts);
-	free(do_mount_opts);
+	free(dev);
 
 	return fd;
 
@@ -1503,65 +1342,6 @@ static void show_version(void)
 	exit(0);
 }
 
-static void close_range_loop(int min_fd, int max_fd, int cfd)
-{
-	for (int fd = min_fd; fd <= max_fd; fd++)
-		if (fd != cfd)
-			close(fd);
-}
-
-/*
- * Close all inherited fds that are not needed
- * Ideally these wouldn't come up at all, applications should better
- * use FD_CLOEXEC / O_CLOEXEC
- */
-static int close_inherited_fds(int cfd)
-{
-	int rc = -1;
-	int nullfd;
-
-	/* We can't even report an error */
-	if (cfd <= STDERR_FILENO)
-		return -EINVAL;
-
-#ifdef HAVE_CLOSE_RANGE
-	if (cfd < STDERR_FILENO + 2) {
-		close_range_loop(STDERR_FILENO + 1, cfd - 1, cfd);
-	} else {
-		rc = close_range(STDERR_FILENO + 1, cfd - 1, 0);
-		if (rc < 0)
-			goto fallback;
-	}
-
-	/* Close high range */
-	rc = close_range(cfd + 1, ~0U, 0);
-#else
-	goto fallback; /* make use of fallback to avoid compiler warnings */
-#endif
-
-fallback:
-	if (rc < 0) {
-		int max_fd = sysconf(_SC_OPEN_MAX) - 1;
-
-		close_range_loop(STDERR_FILENO + 1, max_fd, cfd);
-	}
-
-	nullfd = open("/dev/null", O_RDWR);
-	if (nullfd < 0) {
-		perror("fusermount: cannot open /dev/null");
-		return -errno;
-	}
-
-	/* Redirect stdin, stdout, stderr to /dev/null */
-	dup2(nullfd, STDIN_FILENO);
-	dup2(nullfd, STDOUT_FILENO);
-	dup2(nullfd, STDERR_FILENO);
-	if (nullfd > STDERR_FILENO)
-		close(nullfd);
-
-	return 0;
-}
-
 int main(int argc, char *argv[])
 {
 	sigset_t sigset;
@@ -1573,23 +1353,22 @@ int main(int argc, char *argv[])
 	static int unmount = 0;
 	static int lazy = 0;
 	static int quiet = 0;
-	char *commfd = NULL;
-	long cfd;
+	char *commfd;
+	int cfd;
 	const char *opts = "";
 	const char *type = NULL;
 	int setup_auto_unmount_only = 0;
 
 	static const struct option long_opts[] = {
 		{"unmount", no_argument, NULL, 'u'},
+		// Note: auto-unmount deliberately does not have a short version.
+		// It's meant for internal use by mount.c's setup_auto_unmount.
+		{"auto-unmount", no_argument, NULL, 'U'},
 		{"lazy",    no_argument, NULL, 'z'},
 		{"quiet",   no_argument, NULL, 'q'},
 		{"help",    no_argument, NULL, 'h'},
 		{"version", no_argument, NULL, 'V'},
 		{"options", required_argument, NULL, 'o'},
-		// Note: auto-unmount and comm-fd don't have short versions.
-		// They'ne meant for internal use by mount.c
-		{"auto-unmount", no_argument, NULL, 'U'},
-		{"comm-fd", required_argument, NULL, 'c'},
 		{0, 0, 0, 0}};
 
 	progname = strdup(argc > 0 ? argv[0] : "fusermount");
@@ -1621,9 +1400,6 @@ int main(int argc, char *argv[])
 			auto_unmount = 1;
 			setup_auto_unmount_only = 1;
 			break;
-		case 'c':
-			commfd = optarg;
-			break;
 		case 'z':
 			lazy = 1;
 			break;
@@ -1670,29 +1446,20 @@ int main(int argc, char *argv[])
 	if (!setup_auto_unmount_only && unmount)
 		goto do_unmount;
 
-	if(commfd == NULL)
-		commfd = getenv(FUSE_COMMFD_ENV);
+	commfd = getenv(FUSE_COMMFD_ENV);
 	if (commfd == NULL) {
 		fprintf(stderr, "%s: old style mounting not supported\n",
 			progname);
 		goto err_out;
 	}
 
-	res = libfuse_strtol(commfd, &cfd);
-	if (res) {
-		fprintf(stderr,
-			"%s: invalid _FUSE_COMMFD: %s\n",
-			progname, commfd);
-		goto err_out;
-
-	}
-
+	cfd = atoi(commfd);
 	{
 		struct stat statbuf;
 		fstat(cfd, &statbuf);
 		if(!S_ISSOCK(statbuf.st_mode)) {
 			fprintf(stderr,
-				"%s: file descriptor %li is not a socket, can't send fuse fd\n",
+				"%s: file descriptor %i is not a socket, can't send fuse fd\n",
 				progname, cfd);
 			goto err_out;
 		}
@@ -1721,13 +1488,8 @@ int main(int argc, char *argv[])
 wait_for_auto_unmount:
 	/* Become a daemon and wait for the parent to exit or die.
 	   ie For the control socket to get closed.
-	   Btw, we don't want to use daemon() function here because
+	   btw We don't want to use daemon() function here because
 	   it forks and messes with the file descriptors. */
-
-	res = close_inherited_fds(cfd);
-	if (res < 0)
-		exit(EXIT_FAILURE);
-
 	setsid();
 	res = chdir("/");
 	if (res == -1) {
